package ui;

import java.util.List;

import ui.internal.Maze;

/**
 * Environment class provides interaction with MDP model. It is possible to set
 * probability of correct transition which is set to 0.8 by default. Students
 * will use this class to obtain all states of MDP and they can get reward for
 * each state and transition probability between any two states using some
 * action. This object is given to students.
 * 
 * @author Petr
 */

public class Environment {

	private MDPModel transitionModel;

	public Environment(Maze maze) {
		transitionModel = new MDPModel(maze);
	}

	/**
	 * @param P
	 *            probability of correct transition between two neighbor states
	 *            if there is no walls around a state
	 */

	public void setProbabilityOfCorrectTransition(double P) {
		transitionModel.setPcorrect(P);
	}

	/**
	 * @param state
	 * @return reward in terms of MDP
	 */

	public double getReward(StudentStateInterface state) {
		return transitionModel.getReward(state);
	}

	/**
	 * @return all states in MDP
	 */

	public List<StudentStateInterface> getAllStates() {
		return transitionModel.getAllStates();
	}

	/**
	 * @param fromState
	 * @param action
	 * @param toState
	 * @return transition probability in terms of MDP
	 */

	public double getTransitionProbability(StudentStateInterface fromState,
			Action action, StudentStateInterface toState) {
		return transitionModel.getTransitionProbability(fromState, action,
				toState);
	}
}
