package ui;

/**
 * This is interface for computing values and policies. Let you implement heavy
 * computations in setEnvironment method. Data are read using methods
 * getActionForState and getValueForState by GUI and displayed. Each class
 * implementing this interface must have constructor without arguments. Else the
 * solution cannot be loaded by the main execution class.
 * 
 * @author Petr
 * 
 */

public interface SolutionInterface {

	/**
	 * In this method students obtain environment object. Students should
	 * implement all heavy computations in this method. Students should set
	 * Pcorrect of the environment at the beginning of this method.
	 * 
	 * @param environment
	 * 
	 */

	public void solveMDP(Environment environment);

	/**
	 * This method is called by the system and provides environment to students.
	 * 
	 * @param state
	 * @return action for given state (this is policy in fact). This method
	 *         should be lightweight.
	 */

	public Action getActionForState(StudentStateInterface state);

	/**
	 * @param state
	 * @return value for given state. This method should be lightweight.
	 */

	public double getValueForState(StudentStateInterface state);

}
