package ui.internal;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.zip.GZIPInputStream;
import java.util.zip.GZIPOutputStream;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JSeparator;
import javax.swing.JTextField;
import javax.swing.WindowConstants;
import javax.swing.border.TitledBorder;


import ui.Action;

public class ResourceMazeEditor extends JFrame implements ActionListener {

	/* all the components */
	private JPanel jPanel;
	private JButton jHeight, jWidth, jWalls, jSetReward, jAddGoals, jAddTeleportStates,jSaveMaze,
			jBoxSize, jResetMaze;
	private JTextField jHeightTextField, jWidthTextField, jPenaltyTextField,
			jBoxSizeTextField;
	private MazeViewPanel jGridPanel;
	private JSeparator jSeparator1;

	/* all the listeners */
	private SymMouse aSymMouse;

	/* other objects and variables */
	private ui.internal.Maze myMaze;
	private double reward = 0; // the penalty to be set for the walls

	// walls
	int nodeLength = 40; // indicates the size of the square box to be displayed
	
	float wallWidth = 7;
	
	// in the GUI
	// by default it is 40 pixels. increase to increase magnification factor

	private int edit_state;
	public static final int EDIT_WALLS = 1, ADD_ABSORBING_GOALS = 2, 
			ASSIGN_REWARD = 3, ADD_TELEPORT_STATES = 4;	
	
	private JButton jLoadMaze;
	
	
	
	{
		// Set Look & Feel
		try {
			javax.swing.UIManager
					.setLookAndFeel("com.sun.java.swing.plaf.windows.WindowsLookAndFeel");
		} catch (Exception e) {
			e.printStackTrace();
		}
	}	
	
	public static void main(String[] args) {
		new ResourceMazeEditor();
	}

	public ResourceMazeEditor() {
		super("Maze Editor");
		initGUI();
		setVisible(true);
	}	
	
	
	private void initGUI() {
		try {
			myMaze = new Maze();
			myMaze.init(2, 2, 0);

			setSize(600, 600);

			setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
			aSymMouse = new SymMouse();
			{
				jPanel = new JPanel();
				this.getContentPane().add(jPanel, BorderLayout.CENTER);
				jPanel.setBackground(new java.awt.Color(235, 241, 238));
				jPanel.setLayout(null);

				{
					jHeight = new JButton();
					jPanel.add(jHeight);
					jHeight.setText("Height");
					jHeight.setBounds(5 + 65, 5, 5 + 85, 25);
					jHeight.addActionListener(this);
				}
				{
					jHeightTextField = new JTextField("2");
					jHeightTextField.setBounds(5, 5, 65, 25);
					jPanel.add(jHeightTextField);
				}
				{
					jWidth = new JButton();
					jPanel.add(jWidth);
					jWidth.setText("Width");
					jWidth.setBounds(5 + 65, 30 + 5, 5 + 85, 25);
					jWidth.addActionListener(this);
				}
				{
					jWidthTextField = new JTextField("2");
					jWidthTextField.setBounds(5, 30 + 5, 65, 25);
					jPanel.add(jWidthTextField);
				}
				{
					jSetReward = new JButton();
					jPanel.add(jSetReward);
					jSetReward.setText("Set Reward");
					jSetReward.setBounds(5 + 65, 30 + 30 + 5, 5 + 85, 25);
					jSetReward.addActionListener(this);
				}
				{
					jPenaltyTextField = new JTextField("50");
					jPenaltyTextField
							.setBounds(5, 30 + 30 + 5, 30 + 30 + 5, 25);
					jPanel.add(jPenaltyTextField);
				}
				{
					jWalls = new JButton();
					jPanel.add(jWalls);
					jWalls.setText("Add Walls");
					jWalls.setBounds(5, 30 + 30 + 30 + 5, 5 + 70 + 80, 25);
					jWalls.addActionListener(this);
				}
				{
					jAddGoals = new JButton();
					jPanel.add(jAddGoals);
					jAddGoals.setText("Add Absorbing Goals");
					jAddGoals.setBounds(5, 30 + 30 + 30 + 30 + 5, 5 + 70 + 80,
							25);
					jAddGoals.addActionListener(this);
				}
				{
					jAddTeleportStates = new JButton();
					jPanel.add(jAddTeleportStates);
					jAddTeleportStates.setText("Add Teleport States");
					jAddTeleportStates.setBounds(5, 30 + 30 + 30 + 30 + 30 + 5, 5 + 70 + 80,
							25);
					jAddTeleportStates.addActionListener(this);
				}
				{
					jSaveMaze = new JButton();
					jPanel.add(jSaveMaze);
					jSaveMaze.setText("Save Maze");
					jSaveMaze.setBounds(5, 30 + 30 + 30 + 30 + 30 + 30 + 5,
							5 + 70 + 80, 25);
					jSaveMaze.addActionListener(this);
				}
				{
					jBoxSize = new JButton();
					jPanel.add(jBoxSize);
					jBoxSize.setText("Box Size");
					jBoxSize.setBounds(5 + 70, 30 + 30 + 30 + 30 + 30 + 30 +30 + 5,
							5 + 80, 25);
					jBoxSize.addActionListener(this);
				}
				{
					jBoxSizeTextField = new JTextField("40");
					jBoxSizeTextField.setBounds(5, 30 + 30 + 30 + 30 + 30 + 30 + 30
							+ 5, 65, 25);
					jPanel.add(jBoxSizeTextField);
				}
				{
					jResetMaze = new JButton();
					jPanel.add(jResetMaze);
					jResetMaze.setText("Reset Maze");
					jResetMaze.setBounds(5, 30 + 30 + 30 + 30 + 30 + 30 + 30 + 30
							+ 5, 5 + 70 + 80, 25);
					jResetMaze.addActionListener(this);
				}
				{
					jLoadMaze = new JButton();
					jPanel.add(jLoadMaze);
					jLoadMaze.setText("Load Maze");
					jLoadMaze.setBounds(5, 30 + 30 + 30 + 30 + 30 + 30 + 30 + 30
							+ 30 + 5, 5 + 70 + 80, 25);
					jLoadMaze.addActionListener(this);
				}
				{
					jSeparator1 = new JSeparator();
					jPanel.add(jSeparator1);
					jSeparator1.setBounds(188, 2, 2, 363);
					jSeparator1.setBorder(BorderFactory.createTitledBorder(
							null, "", TitledBorder.LEADING, TitledBorder.TOP,
							new java.awt.Font("MS Sans Serif", 0, 11),
							new java.awt.Color(0, 0, 0)));
				}
				{
					jGridPanel = new MazeViewPanel();
					jGridPanel.setMaze(myMaze);
					jPanel.add(jGridPanel);
					jGridPanel.addMouseListener(aSymMouse);
				}
			}

		} catch (Exception e) {
			e.printStackTrace();
		}
	}	
	
	
	class SymMouse extends java.awt.event.MouseAdapter {
		public void mouseClicked(java.awt.event.MouseEvent event) {
			Object object = event.getSource();
			int clickX = event.getX();
			int clickY = event.getY();
			int nodeX = (clickX / nodeLength);
			int nodeY = (clickY / nodeLength);

			if (edit_state == EDIT_WALLS) {
				int lowerLimX = (nodeX * nodeLength) + (int) (0.1 * nodeLength);
				int upperLimX = ((nodeX + 1) * nodeLength)
						- (int) (0.1 * nodeLength);
				if (clickX < lowerLimX) {
					boolean prev = myMaze.isWall(nodeX, nodeY, Action.GO_WEST);
					myMaze.setWall(nodeX, nodeY, Action.GO_WEST, !prev);
				} else if (clickX > upperLimX) {
					boolean prev = myMaze.isWall(nodeX, nodeY, Action.GO_EAST);
					myMaze.setWall(nodeX, nodeY, Action.GO_EAST, !prev);
				}

				int lowerLimY = (nodeY * nodeLength) + (int) (0.1 * nodeLength);
				int upperLimY = ((nodeY + 1) * nodeLength)
						- (int) (0.1 * nodeLength);
				if (clickY < lowerLimY) {
					boolean prev = myMaze.isWall(nodeX, nodeY, Action.GO_NORTH);
					myMaze.setWall(nodeX, nodeY, Action.GO_NORTH, !prev);
				} else if (clickY > upperLimY) {
					boolean prev = myMaze.isWall(nodeX, nodeY, Action.GO_SOUTH);
					myMaze.setWall(nodeX, nodeY, Action.GO_SOUTH, !prev);
				}

				repaint();		
			
				return;
			}
			
			if (edit_state == ADD_ABSORBING_GOALS) {
				if (!myMaze.isTeleportState(nodeX, nodeY)) {
					boolean wasAbsorbingGoal = myMaze.isAbsorbingGoal(nodeX,
							nodeY);
					myMaze.setAbsorbingGoal(nodeX, nodeY, !wasAbsorbingGoal);
					repaint();
				}
				return;
			}
			
			if (edit_state == ADD_TELEPORT_STATES) {
				if (!myMaze.isAbsorbingGoal(nodeX, nodeY)) {
					boolean wasTeleportState = myMaze.isTeleportState(nodeX,
							nodeY);
					myMaze.setTeleportState(nodeX, nodeY, !wasTeleportState);
					repaint();
				}
				
				return;
			}
				
			
			if (edit_state == ASSIGN_REWARD) {
				myMaze.setReward(nodeX, nodeY, reward);
				repaint();
				
				return; 
			}
			
		}
	}	
	
	
	
	public void actionPerformed(ActionEvent evt) {
		// System.out.println("event="+ evt);
		if (evt.getSource() == jLoadMaze) {
			loadFile();
			repaint();
		} else if (evt.getSource() == jHeight) {
			int a = Integer.parseInt(jHeightTextField.getText());
			int oldWidth = myMaze.getWidth();

			myMaze.init(oldWidth, a, 0);
			repaint();
		} else if (evt.getSource() == jWidth) {
			int a = Integer.parseInt(jWidthTextField.getText());
			int oldHeight = myMaze.getHeight();
			myMaze.init(a, oldHeight, 0);
			repaint();
		} else if (evt.getSource() == jWalls) {
			edit_state = EDIT_WALLS;
			repaint();
		} else if (evt.getSource() == jSetReward) {
			reward = Double.parseDouble(jPenaltyTextField.getText());
			edit_state = ASSIGN_REWARD;
		} else if (evt.getSource() == jAddGoals) {
			edit_state = ADD_ABSORBING_GOALS;
		} else if (evt.getSource() == jAddTeleportStates) {
			edit_state = ADD_TELEPORT_STATES;
		} else if (evt.getSource() == jSaveMaze) {
			saveFile();
		} else if (evt.getSource() == jBoxSize) {
			nodeLength = Integer.parseInt(jBoxSizeTextField.getText());
			//System.out.println("new node lenght "+nodeLength);
			jGridPanel.setCellSize(nodeLength);
			
		} else if (evt.getSource() == jResetMaze) {
			myMaze.init(2, 2, 0);
			repaint();
		}

	}
	
	
	
	private void loadFile() {
		JFileChooser fc = new JFileChooser("./mazes/");
		int returnVal = fc.showOpenDialog(ResourceMazeEditor.this);
		if (returnVal == JFileChooser.APPROVE_OPTION) {
			try {
				File file = fc.getSelectedFile();
				FileInputStream fis = new FileInputStream(file);
				GZIPInputStream gzis = new GZIPInputStream(fis);
				ObjectInputStream in = new ObjectInputStream(gzis);

				myMaze.setMazeData((MazeData) in.readObject());

				in.close();
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
	}

	private void saveFile() {
		JFileChooser fc = new JFileChooser("./mazes/");
		int returnVal = fc.showOpenDialog(ResourceMazeEditor.this);
		if (returnVal == JFileChooser.APPROVE_OPTION) {
			try {
				File file = fc.getSelectedFile();
				FileOutputStream fos = new FileOutputStream(file);
				GZIPOutputStream gzos = new GZIPOutputStream(fos);
				ObjectOutputStream out = new ObjectOutputStream(gzos);

				out.writeObject(myMaze.getMazeData());

				out.flush();
				out.close();
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
	}	

}
