package ui.internal;

import java.awt.BorderLayout;
import java.awt.Checkbox;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.io.File;
import java.io.FileInputStream;
import java.io.ObjectInputStream;
import java.util.zip.GZIPInputStream;

import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JTextField;

import ui.Environment;


public class SolutionViewer extends JFrame implements ActionListener, ItemListener {

	private SolutionViewPanel solutionPanel;
	
	private Checkbox drawActionsCheckbox;
	private Checkbox drawValuesCheckbox; 
	private Checkbox drawGoalsCheckbox;
	private Checkbox drawRewardsCheckbox;
	private Checkbox drawTeleportsCheckbox;
	private Checkbox drawWallsCheckbox;
	
	
	private JButton cellSizeButton; 	
	private JTextField cellSizeTextField; 
	
	private Maze myMaze = new Maze();
	private Environment environment;
	

	JButton loadButton;
	
	public SolutionViewer(SolutionViewPanel solutionPanel) {
		super("MDP solution viewer");
		this.solutionPanel = solutionPanel;
		JPanel panel = new JPanel(new BorderLayout());
		this.add(panel);
		
		JPanel panel2 = new JPanel(new GridBagLayout()) ; 

		GridBagConstraints c0 = new GridBagConstraints();
		c0.anchor = GridBagConstraints.LINE_START;
		c0.gridx = 0;
		c0.gridy = 0;
		loadButton = new JButton("Load maze");
		panel2.add(loadButton, c0);
		
		
		loadButton.addActionListener(this);
		
		
		GridBagConstraints c1 = new GridBagConstraints();
		c1.anchor = GridBagConstraints.LINE_START;
		c1.gridx = 0;
		c1.gridy = 1;
		drawActionsCheckbox = new Checkbox("draw actions");
		panel2.add(drawActionsCheckbox, c1);
		drawActionsCheckbox.setState(solutionPanel.drawActions);
		drawActionsCheckbox.addItemListener(this);
		
		
		
		GridBagConstraints c2 = new GridBagConstraints();
		c2.anchor = GridBagConstraints.LINE_START;
		c2.gridx = 0;
		c2.gridy = 2;
		drawValuesCheckbox = new Checkbox("draw values");
		panel2.add(drawValuesCheckbox, c2);
		drawValuesCheckbox.setState(solutionPanel.drawValues);
		drawValuesCheckbox.addItemListener(this);
		
		
		GridBagConstraints c3 = new GridBagConstraints();
		c3.anchor = GridBagConstraints.LINE_START;
		c3.gridx = 0;
		c3.gridy = 3;
		drawGoalsCheckbox = new Checkbox("draw goals");
		panel2.add(drawGoalsCheckbox, c3);
		drawGoalsCheckbox.setState(solutionPanel.drawGoals);
		drawGoalsCheckbox.addItemListener(this);
		
		
		GridBagConstraints c4 = new GridBagConstraints();
		c4.anchor = GridBagConstraints.LINE_START;
		c4.gridx = 0;
		c4.gridy = 4;
		drawRewardsCheckbox = new Checkbox("draw rewards");
		panel2.add(drawRewardsCheckbox, c4);
		drawRewardsCheckbox.setState(solutionPanel.drawRewards);
		drawRewardsCheckbox.addItemListener(this);
		
		GridBagConstraints c5 = new GridBagConstraints();
		c5.anchor = GridBagConstraints.LINE_START;
		c5.gridx = 0;
		c5.gridy = 5;
		drawTeleportsCheckbox = new Checkbox("draw teleports");
		panel2.add(drawTeleportsCheckbox, c5);
		drawTeleportsCheckbox.setState(solutionPanel.drawTeleports);
		drawTeleportsCheckbox.addItemListener(this);
		
		GridBagConstraints c6 = new GridBagConstraints();
		c6.anchor = GridBagConstraints.LINE_START;
		c6.gridx = 0;
		c6.gridy = 6;
		drawWallsCheckbox = new Checkbox("draw walls");
		panel2.add(drawWallsCheckbox, c6); 
		drawWallsCheckbox.setState(solutionPanel.drawWalls);
		drawWallsCheckbox.addItemListener(this);
		
		GridBagConstraints c7 = new GridBagConstraints();
		c7.anchor = GridBagConstraints.LINE_START;
		c7.gridx = 0;
		c7.gridy = 7;
		cellSizeTextField = new JTextField("40");
		cellSizeTextField.setColumns(5);
		
		panel2.add(cellSizeTextField, c7);
		

		GridBagConstraints c8 = new GridBagConstraints();
		c8.anchor = GridBagConstraints.LINE_START;
		c8.gridx = 0;
		c8.gridy = 8;
		cellSizeButton = new JButton("set cell size");
		panel2.add(cellSizeButton, c8);
		
		cellSizeButton.addActionListener(this);
		
		panel.add(panel2, BorderLayout.WEST);
		panel.add(solutionPanel, BorderLayout.CENTER);
		this.setSize(600, 600);
		this.setVisible(true);
	}
	
	
	
	public void actionPerformed(ActionEvent evt) {
		if (evt.getSource() == loadButton) {
			loadFile();
			solutionPanel.setMaze(myMaze);
		}
		
		if (evt.getSource() == cellSizeButton) {
			 int cellSize = Integer.parseInt(cellSizeTextField.getText());
			solutionPanel.setCellSize(cellSize);
		}

				
		
	}
		
	private void loadFile() {
		JFileChooser fc = new JFileChooser("./mazes/");
		int returnVal = fc.showOpenDialog(SolutionViewer.this);
		if (returnVal == JFileChooser.APPROVE_OPTION) {
			try {
				File file = fc.getSelectedFile();
				FileInputStream fis = new FileInputStream(file);
				GZIPInputStream gzis = new GZIPInputStream(fis);
				ObjectInputStream in = new ObjectInputStream(gzis);

				myMaze.setMazeData((MazeData) in.readObject());
				environment = new Environment(myMaze);
				solutionPanel.setEnvironment(environment);				

				in.close();
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
	}	
	

	@Override
	public void itemStateChanged(ItemEvent arg0) {
		
		if (arg0.getSource() == drawActionsCheckbox) {
			 boolean drawActions = drawActionsCheckbox.getState();
			 solutionPanel.setDrawActions(drawActions);
		}
		
		if (arg0.getSource() == drawValuesCheckbox) {
			 boolean drawValues = drawValuesCheckbox.getState();
			 solutionPanel.setDrawValues(drawValues);
		}
		
		if (arg0.getSource() == drawGoalsCheckbox) {
			 boolean drawGoals = drawGoalsCheckbox.getState();
			 solutionPanel.setDrawGoals(drawGoals);			
		}

		
		if (arg0.getSource() == drawRewardsCheckbox) {
			 boolean drawRewards = drawRewardsCheckbox.getState();
			 solutionPanel.setDrawRewards(drawRewards);			
		}
		
		if (arg0.getSource() == drawTeleportsCheckbox) {
			 boolean teleports = drawTeleportsCheckbox.getState();
			 solutionPanel.setDrawTeleports(teleports);						
		}
		
		if (arg0.getSource() == drawWallsCheckbox) {
			 boolean teleports = drawWallsCheckbox.getState();
			 solutionPanel.setDrawWalls(teleports);						
		}
		

		
	}
	
	

}
